
/*! 
	\file quatre.c
	\author GOIGOUX Lucie & JEUDI--LEMOINE Alix
	\date 12/01/23
	\brief Partie 4 de la SAE 1.02

	Application de gestion des candidature dans les IUT de France
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <strings.h> // strcasecmp : insensitive case
#include <unistd.h> // access : vérifier si fichier existe
#include "quatre.h"


void menuCandidat_suivreCandidature(ListeCandidats liste, int nbCandidats, Candidat* candidat) {
	int i, j, mustReply = 0, nbPropositions = 0;
	char reponse1, reponse2, poubelle;

	// Création d'une liste qui va contenir les candidatures pour lesquelles le candidat a été accepté
	// (pour le moment, seul une proposition peut être faite étant donné que seul le département Informatique de Clermont-Ferrand est habilité à donner une réponse)
	ListeCandidatures propositions = (ListeCandidatures) malloc(candidat->nbCandidatures*sizeof(Candidature*));

	if(propositions == NULL) {
		perror("malloc");
		exit(errno);
	}

	if(candidat->nbCandidatures == 0)
		fprintf(stderr, "\e[0;91mErreur : vous n'avez aucune candidature en cours, merci de réessayer plus tard.\n\e[0m");
	else
		afficherCandidatures(candidat->listeCandidatures, candidat->nbCandidatures);

	for(i=0; i<candidat->nbCandidatures; i++)
		if(candidat->listeCandidatures[i]->decision == 1 && candidat->listeCandidatures[i]->validation == 0) {
			mustReply = 1;
			propositions[nbPropositions++] = candidat->listeCandidatures[i];
		}
	
	if(mustReply == 1) {
		printf("Vous avez %d candidature(s) contenant une proposition d'admission, souhaitez-vous y répondre ? (O/N) : ", nbPropositions);
		scanf("%*c%c", &reponse1);

		for(i=0; i<nbPropositions; i++) {
			if(reponse1 == 'o' || reponse1 == 'O') {
				afficherCandidatures(propositions+i, 1);

				printf("\nSouhaitez-vous accepter ou refuser cette proposition ? (A/R) : ");
				scanf("%*c%c", &reponse2);
				
				if(reponse2 == 'a' || reponse2 == 'A'){
					printf("\n\e[1;32mProposition pour le département '%s' à l'IUT '%s' acceptée !\e[0m\n",
						   propositions[i]->departement, propositions[i]->ville);

					for(j=0; j<candidat->nbCandidatures; j++) {
						candidat->listeCandidatures[j]->decision = -1;
						candidat->listeCandidatures[j]->validation = -1;
					}

					propositions[i]->decision = 1;
					propositions[i]->validation = 1;

					break;
				} else {
					if(reponse2 == 'r' || reponse2 == 'R')
						printf("\n\e[1;32mProposition pour le département '%s' à l'IUT '%s' refusée !\e[0m\n",
							   propositions[i]->departement, propositions[i]->ville);
					else {
						while ((poubelle = getchar()) != '\n' && poubelle != EOF);
						fprintf(stderr, "\n\e[0;91mErreur : la réponse saisie est incorrecte. Merci de réessayer plus tard.\n\e[0m");
						break;
					}
				}
			} else {
				if(reponse1 == 'n' || reponse1 == 'N')
					printf("\n\t\e[1mTrès bien. Vous pouvez toujours donner votre réponse ultérieurement.\e[0m\n");
				else {
					while ((poubelle = getchar()) != '\n' && poubelle != EOF);
					fprintf(stderr, "\n\e[0;91mErreur : la réponse saisie est incorrecte. Merci de réessayer plus tard.\n\e[0m");
					break;
				}
			}
		}
	}

	free(propositions);
}

void menuResponsable_traiterCandidatures(VilleIUT** tiut, int nbVilles, ListeCandidats liste, int nbCandidats) {
	char dept[LEN_MAX], ville[LEN_MAX], choix, poubelle;
	int nbCandidatsAccept, pos, verif;
	ListeDept ldept;

	strcpy(dept, "Informatique");
	strcpy(ville, "Clermont-Ferrand");

	printf("\n\e[3;32mLes deux prochaines lignes sont affichées à titre informatif\e[0m\n\n");
	printf("Entrez le nom de la ville que vous souhaitez traiter : %s\n", ville);
	printf("Quel département souhaitez-vous modifier ? : %s\n", dept);

	printf("\n\e[1;32mTraitement des candidatures pour le département '%s' de l'IUT '%s'\n\n\e[0m", dept, ville);

	printf("Combien de candidats souhaitez-vous accepter ? : ");
	if(scanf("%d", &nbCandidatsAccept) == 0) {
		while ((poubelle = getchar()) != '\n' && poubelle != EOF);
		fprintf(stderr, "\n\e[0;91mErreur : la valeur entrée est invalide, merci de réessayer plus tard.\n\n\e[0m");
		return;
	}

	pos = rechercherVille(tiut, nbVilles, ville);

	ldept = tiut[pos]->ldept;

    while(ldept != NULL && strcmp(ldept->departement, dept) != 0)
        ldept = ldept->suiv;

    if(ldept->noteMinimale == -1) {
		printf("\n\e[1;91mAttention, après avoir entré cette valeur, elle ne sera plus modifiable !\e[0m\n\nQuelle est la note minimale pour l'acception du candidat ? : ");
		verif = scanf("%f", &ldept->noteMinimale);
		while((ldept->noteMinimale) < 0 || (ldept->noteMinimale) > 20 || verif == 0) {
			fprintf(stderr, "\e[0;91mLa note entrée est incorrecte\n\e[0mEntrez une note minimale entre 0 et 20 : ");
			verif = scanf("%f", &ldept->noteMinimale);
		}
    }

	printf("\nMerci de confirmer : Il faut accepter %d candidat avec une note minimale de %.2f (O/N) ? : ", nbCandidatsAccept, ldept->noteMinimale);

	scanf("%*c%c", &choix);

	if(choix == 'n' || choix == 'N')
		printf("\n\e[1mAnnulation de l'opération.\e[0m\n\n");
	else
		if(choix == 'o' || choix == 'O')
			traiterCandidatures(liste, nbCandidats, nbCandidatsAccept, ldept->noteMinimale);
		else {
			while ((poubelle = getchar()) != '\n' && poubelle != EOF);
			fprintf(stderr, "\n\e[0;91mErreur : la valeur entrée est invalide, merci de réessayer plus tard.\n\n\e[0m");		
		}

}

FileAttente* creerFile(void) {
	FileAttente* file = (FileAttente*) malloc(sizeof(FileAttente));

	if(file == NULL) {
		perror("malloc");
		exit(errno);
	}

    file->tete = NULL;
    file->queue = NULL;

	return file;
}

int estVide(FileAttente *file) {
    return file->tete == NULL;
}

void enfiler(FileAttente *file, CandidatTraite candidat) {
    Maillon* nouveauMaillon = (Maillon*) malloc(sizeof(Maillon));
    nouveauMaillon->candidat = candidat;
    nouveauMaillon->suivant = NULL;

    if (estVide(file)) {
		file->tete = nouveauMaillon;
		file->queue = nouveauMaillon;
	} else
        file->queue->suivant = nouveauMaillon;
}

CandidatTraite defiler(FileAttente *file) {
	CandidatTraite candidat = {"", "", 0, 0, 0, 0, 0};
	Maillon *tete;

    if (!estVide(file)) {
        tete = file->tete;

        candidat = tete->candidat;
        file->tete = tete->suivant;
        free(tete);
    }
	
	return candidat;
}

int compareCandidatsTraite(const void* p1, const void* p2) {
    CandidatTraite* c1 = *(ListeAdmis) p1;
    CandidatTraite* c2 = *(ListeAdmis) p2;

    char nomComplet1[60];
    strcpy(nomComplet1, c1->nom);
    strcat(nomComplet1, c1->prenom);

    char nomComplet2[60];
    strcpy(nomComplet2, c2->nom);
    strcat(nomComplet2, c2->prenom);

    return strcmp(nomComplet1, nomComplet2);
}

void afficherCandidatsAdmis(char* dept, char* ville) {
	ListeAdmis liste;
	int nbAdmis, i;

	char fNameAdmis[100];

	strcpy(fNameAdmis, "donnees/");
    strcat(fNameAdmis, ville);
    strcat(fNameAdmis, "_");
    strcat(fNameAdmis, dept);
    strcat(fNameAdmis, "_Admis.don");

	if(access(fNameAdmis, F_OK) != -1) {
		FILE *fAdmis = fopen(fNameAdmis, "r");

		if(fAdmis == NULL) {
			perror("fopen");
			exit(errno);
		}

		fscanf(fAdmis, "%d\n", &nbAdmis);

		liste = (ListeAdmis) malloc(nbAdmis*sizeof(CandidatTraite*));

		if(liste == NULL) {
			perror("malloc");
			exit(errno);
		}
		
		for(i=0; i<nbAdmis; i++) {
			liste[i] = (CandidatTraite*) malloc(sizeof(CandidatTraite));

			if(liste[i] == NULL) {
				perror("malloc");
				exit(errno);
			}

			fscanf(fAdmis, "%[^\n]\n%[^\n]\n%f %f %f %f %f\n",
				liste[i]->nom, liste[i]->prenom, &liste[i]->moyenneMath, &liste[i]->moyenneFrancais,
				&liste[i]->moyenneAnglais, &liste[i]->moyenneSpecialite, &liste[i]->noteGlobale);
		}
		
		qsort(liste, nbAdmis, sizeof(CandidatTraite*), compareCandidatsTraite);

		for(i=0; i<nbAdmis; i++) {
			printf("\e[4;37mCandidat '%s %s' :\e[0m"
				"\n  - Moyenne en mathématiques : %.2f\n  - Moyenne en français : %.2f"
				"\n  - Moyenne en anglais : %.2f\n  - Moyenne en spécialité : %.2f"
				"\n  - Note globale attribuée : %.2f\n\n",
				liste[i]->nom, liste[i]->prenom, liste[i]->moyenneMath, liste[i]->moyenneFrancais,
				liste[i]->moyenneAnglais, liste[i]->moyenneSpecialite, liste[i]->noteGlobale);
		}

		fclose(fAdmis);
		free(liste);
	} else {
		fprintf(stderr, "\n\e[1;91mErreur : les candidatures n'ont pas encore été traitées, merci de réessayer plus tard.\n\n\e[0m");
	}
}